@@ -19,7 +19,7 @@
  * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
  * Boston, MA 02110-1301 USA.
  *
- * (C) Copyright 2007 - 2010 Red Hat, Inc.
+ * (C) Copyright 2007 - 2011 Red Hat, Inc.
  * (C) Copyright 2007 - 2008 Novell, Inc.
  */
 
@@ -27,24 +27,8 @@
 #include <ctype.h>
 #include "nm-setting-gsm.h"
 #include "nm-utils.h"
+#include "nm-setting-private.h"
 
-/**
- * SECTION:nm-setting-gsm
- * @short_description: Describes GSM/3GPP-based mobile broadband properties
- * @include: nm-setting-gsm.h
- *
- * The #NMSettingGsm object is a #NMSetting subclass that describes
- * properties that allow connections to 3GPP-based mobile broadband
- * networks, including those using GPRS/EDGE and UMTS/HSPA technology.
- */
-
-/**
- * nm_setting_gsm_error_quark:
- *
- * Registers an error quark for #NMSettingGsm if necessary.
- *
- * Returns: the error quark used for #NMSettingGsm errors.
- **/
 GQuark
 nm_setting_gsm_error_quark (void)
 {
@@ -89,6 +73,7 @@ typedef struct {
 	char *number; /* For dialing, duh */
 	char *username;
 	char *password;
+	NMSettingSecretFlags password_flags;
 
 	char *apn; /* NULL for dynamic */
 	char *network_id; /* for manual registration or NULL for automatic */
@@ -96,6 +81,7 @@ typedef struct {
 	guint32 allowed_bands;     /* A bitfield of NM_SETTING_GSM_BAND_* */
 
 	char *pin;
+	NMSettingSecretFlags pin_flags;
 
 	gboolean home_only;
 } NMSettingGsmPrivate;
@@ -105,37 +91,24 @@ enum {
 	PROP_NUMBER,
 	PROP_USERNAME,
 	PROP_PASSWORD,
+	PROP_PASSWORD_FLAGS,
 	PROP_APN,
 	PROP_NETWORK_ID,
 	PROP_NETWORK_TYPE,
-	PROP_BAND,
 	PROP_PIN,
-	PROP_PUK,
+	PROP_PIN_FLAGS,
 	PROP_ALLOWED_BANDS,
 	PROP_HOME_ONLY,
 
 	LAST_PROP
 };
 
-/**
- * nm_setting_gsm_new:
- *
- * Creates a new #NMSettingGsm object with default values.
- *
- * Returns: the new empty #NMSettingGsm object
- **/
 NMSetting *
 nm_setting_gsm_new (void)
 {
 	return (NMSetting *) g_object_new (NM_TYPE_SETTING_GSM, NULL);
 }
 
-/**
- * nm_setting_gsm_get_number:
- * @setting: the #NMSettingGsm
- *
- * Returns: the #NMSettingGsm:number property of the setting
- **/
 const char *
 nm_setting_gsm_get_number (NMSettingGsm *setting)
 {
@@ -144,12 +117,6 @@ nm_setting_gsm_get_number (NMSettingGsm *setting)
 	return NM_SETTING_GSM_GET_PRIVATE (setting)->number;
 }
 
-/**
- * nm_setting_gsm_get_username:
- * @setting: the #NMSettingGsm
- *
- * Returns: the #NMSettingGsm:username property of the setting
- **/
 const char *
 nm_setting_gsm_get_username (NMSettingGsm *setting)
 {
@@ -158,12 +125,6 @@ nm_setting_gsm_get_username (NMSettingGsm *setting)
 	return NM_SETTING_GSM_GET_PRIVATE (setting)->username;
 }
 
-/**
- * nm_setting_gsm_get_password:
- * @setting: the #NMSettingGsm
- *
- * Returns: the #NMSettingGsm:password property of the setting
- **/
 const char *
 nm_setting_gsm_get_password (NMSettingGsm *setting)
 {
@@ -173,11 +134,19 @@ nm_setting_gsm_get_password (NMSettingGsm *setting)
 }
 
 /**
- * nm_setting_gsm_get_apn:
+ * nm_setting_gsm_get_password_flags:
  * @setting: the #NMSettingGsm
  *
- * Returns: the #NMSettingGsm:apn property of the setting
+ * Returns: the #NMSettingSecretFlags pertaining to the #NMSettingGsm:password
  **/
+NMSettingSecretFlags
+nm_setting_gsm_get_password_flags (NMSettingGsm *setting)
+{
+	g_return_val_if_fail (NM_IS_SETTING_GSM (setting), NM_SETTING_SECRET_FLAG_NONE);
+
+	return NM_SETTING_GSM_GET_PRIVATE (setting)->password_flags;
+}
+
 const char *
 nm_setting_gsm_get_apn (NMSettingGsm *setting)
 {
@@ -186,12 +155,6 @@ nm_setting_gsm_get_apn (NMSettingGsm *setting)
 	return NM_SETTING_GSM_GET_PRIVATE (setting)->apn;
 }
 
-/**
- * nm_setting_gsm_get_network_id:
- * @setting: the #NMSettingGsm
- *
- * Returns: the #NMSettingGsm:network-id property of the setting
- **/
 const char *
 nm_setting_gsm_get_network_id (NMSettingGsm *setting)
 {
@@ -200,12 +163,6 @@ nm_setting_gsm_get_network_id (NMSettingGsm *setting)
 	return NM_SETTING_GSM_GET_PRIVATE (setting)->network_id;
 }
 
-/**
- * nm_setting_gsm_get_network_type:
- * @setting: the #NMSettingGsm
- *
- * Returns: the #NMSettingGsm:network-type property of the setting
- **/
 int
 nm_setting_gsm_get_network_type (NMSettingGsm *setting)
 {
@@ -214,19 +171,6 @@ nm_setting_gsm_get_network_type (NMSettingGsm *setting)
 	return NM_SETTING_GSM_GET_PRIVATE (setting)->network_type;
 }
 
-int
-nm_setting_gsm_get_band (NMSettingGsm *setting)
-{
-	g_warning ("Tried to get deprecated property " NM_SETTING_GSM_SETTING_NAME "/" NM_SETTING_GSM_BAND);
-	return -1;
-}
-
-/**
- * nm_setting_gsm_get_allowed_bands:
- * @setting: the #NMSettingGsm
- *
- * Returns: the #NMSettingGsm:allowed-bands property of the setting
- **/
 guint32
 nm_setting_gsm_get_allowed_bands (NMSettingGsm *setting)
 {
@@ -235,12 +179,6 @@ nm_setting_gsm_get_allowed_bands (NMSettingGsm *setting)
 	return NM_SETTING_GSM_GET_PRIVATE (setting)->allowed_bands;
 }
 
-/**
- * nm_setting_gsm_get_pin:
- * @setting: the #NMSettingGsm
- *
- * Returns: the #NMSettingGsm:pin property of the setting
- **/
 const char *
 nm_setting_gsm_get_pin (NMSettingGsm *setting)
 {
@@ -249,19 +187,20 @@ nm_setting_gsm_get_pin (NMSettingGsm *setting)
 	return NM_SETTING_GSM_GET_PRIVATE (setting)->pin;
 }
 
-const char *
-nm_setting_gsm_get_puk (NMSettingGsm *setting)
-{
-	g_warning ("Tried to get deprecated property " NM_SETTING_GSM_SETTING_NAME "/" NM_SETTING_GSM_PUK);
-	return NULL;
-}
-
 /**
- * nm_setting_gsm_get_home_only:
+ * nm_setting_gsm_get_pin_flags:
  * @setting: the #NMSettingGsm
  *
- * Returns: the #NMSettingGsm:home-only property of the setting
+ * Returns: the #NMSettingSecretFlags pertaining to the #NMSettingGsm:pin
  **/
+NMSettingSecretFlags
+nm_setting_gsm_get_pin_flags (NMSettingGsm *setting)
+{
+	g_return_val_if_fail (NM_IS_SETTING_GSM (setting), NM_SETTING_SECRET_FLAG_NONE);
+
+	return NM_SETTING_GSM_GET_PRIVATE (setting)->pin_flags;
+}
+
 gboolean
 nm_setting_gsm_get_home_only (NMSettingGsm *setting)
 {
@@ -302,28 +241,10 @@ verify (NMSetting *setting, GSList *all_settings, GError **error)
 		}
 
 		/* APNs roughly follow the same rules as DNS domain names.  Allowed
-		 * characters are a-z, 0-9, . and -.  GSM 03.03 Section 9.1 states:
-		 *
-		 *   The syntax of the APN shall follow the Name Syntax defined in
-		 *   RFC 2181 [14] and RFC 1035 [15]. The APN consists of one or
-		 *   more labels. Each label is coded as one octet length field
-		 *   followed by that number of octets coded as 8 bit ASCII characters.
-		 *   Following RFC 1035 [15] the labels should consist only of the
-		 *   alphabetic characters (A-Z and a-z), digits (0-9) and the
-		 *   dash (-). The case of alphabetic characters is not significant.
-		 *
-		 * A dot (.) is commonly used to separate parts of the APN, and
-		 * apparently the underscore (_) is used as well.  RFC 2181 indicates
-		 * that no restrictions of any kind are placed on DNS labels, and thus
-		 * it would appear that none are placed on APNs either, but many modems
-		 * and networks will fail to accept APNs that include odd characters
-		 * like space ( ) and such.
+		 * characters are a-z, 0-9, . and -.  GSM 03.60 Section 14.9.
 		 */
 		for (i = 0; i < apn_len; i++) {
-			if (   !isalnum (priv->apn[i])
-			    && (priv->apn[i] != '.')
-			    && (priv->apn[i] != '_')
-			    && (priv->apn[i] != '-')) {
+			if (!isalnum (priv->apn[i]) && (priv->apn[i] != '.') && (priv->apn[i] != '-')) {
 				g_set_error (error,
 				             NM_SETTING_GSM_ERROR,
 				             NM_SETTING_GSM_ERROR_INVALID_PROPERTY,
@@ -386,8 +307,10 @@ need_secrets (NMSetting *setting)
 		return NULL;
 
 	if (priv->username) {
-		secrets = g_ptr_array_sized_new (1);
-		g_ptr_array_add (secrets, NM_SETTING_GSM_PASSWORD);
+		if (!(priv->password_flags & NM_SETTING_SECRET_FLAG_NOT_REQUIRED)) {
+			secrets = g_ptr_array_sized_new (1);
+			g_ptr_array_add (secrets, NM_SETTING_GSM_PASSWORD);
+		}
 	}
 
 	return secrets;
@@ -419,7 +342,6 @@ set_property (GObject *object, guint prop_id,
 		    const GValue *value, GParamSpec *pspec)
 {
 	NMSettingGsmPrivate *priv = NM_SETTING_GSM_GET_PRIVATE (object);
-	const char *str;
 	char *tmp;
 
 	switch (prop_id) {
@@ -435,6 +357,9 @@ set_property (GObject *object, guint prop_id,
 		g_free (priv->password);
 		priv->password = g_value_dup_string (value);
 		break;
+	case PROP_PASSWORD_FLAGS:
+		priv->password_flags = g_value_get_uint (value);
+		break;
 	case PROP_APN:
 		g_free (priv->apn);
 		priv->apn = NULL;
@@ -452,10 +377,6 @@ set_property (GObject *object, guint prop_id,
 	case PROP_NETWORK_TYPE:
 		priv->network_type = g_value_get_int (value);
 		break;
-	case PROP_BAND:
-		if (g_value_get_int (value) != -1)
-			g_warning ("Tried to set deprecated property " NM_SETTING_GSM_SETTING_NAME "/" NM_SETTING_GSM_BAND);
-		break;
 	case PROP_ALLOWED_BANDS:
 		priv->allowed_bands = g_value_get_uint (value);
 		break;
@@ -463,10 +384,8 @@ set_property (GObject *object, guint prop_id,
 		g_free (priv->pin);
 		priv->pin = g_value_dup_string (value);
 		break;
-	case PROP_PUK:
-		str = g_value_get_string (value);
-		if (str && strlen (str))
-			g_warning ("Tried to set deprecated property " NM_SETTING_GSM_SETTING_NAME "/" NM_SETTING_GSM_PUK);
+	case PROP_PIN_FLAGS:
+		priv->pin_flags = g_value_get_uint (value);
 		break;
 	case PROP_HOME_ONLY:
 		priv->home_only = g_value_get_boolean (value);
@@ -493,6 +412,9 @@ get_property (GObject *object, guint prop_id,
 	case PROP_PASSWORD:
 		g_value_set_string (value, nm_setting_gsm_get_password (setting));
 		break;
+	case PROP_PASSWORD_FLAGS:
+		g_value_set_uint (value, nm_setting_gsm_get_password_flags (setting));
+		break;
 	case PROP_APN:
 		g_value_set_string (value, nm_setting_gsm_get_apn (setting));
 		break;
@@ -508,13 +430,8 @@ get_property (GObject *object, guint prop_id,
 	case PROP_PIN:
 		g_value_set_string (value, nm_setting_gsm_get_pin (setting));
 		break;
-	case PROP_PUK:
-		/* deprecated */
-		g_value_set_string (value, NULL);
-		break;
-	case PROP_BAND:
-		/* deprecated */
-		g_value_set_int (value, -1);
+	case PROP_PIN_FLAGS:
+		g_value_set_uint (value, nm_setting_gsm_get_pin_flags (setting));
 		break;
 	case PROP_HOME_ONLY:
 		g_value_set_boolean (value, nm_setting_gsm_get_home_only (setting));
@@ -546,22 +463,21 @@ nm_setting_gsm_class_init (NMSettingGsmClass *setting_class)
 	 * NMSettingGsm:number:
 	 *
 	 * Number to dial when establishing a PPP data session with the GSM-based
-	 * mobile broadband network.  Many modems do not require PPP for connections
-	 * to the mobile network and thus this property should be left blank, which
-	 * allows NetworkManager to select the appropriate settings automatically.
+	 * mobile broadband network.  In most cases, leave the number blank and a
+	 * number selecting the APN specified in the 'apn' property will be used
+	 * automatically when required.
 	 **/
 	g_object_class_install_property
 		(object_class, PROP_NUMBER,
 		 g_param_spec_string (NM_SETTING_GSM_NUMBER,
-		                      "Number",
-		                      "Number to dial when establishing a PPP data session "
-		                      "with the GSM-based mobile broadband network.  Many "
-		                      "modems do not require PPP for connections to the "
-		                      "mobile network and thus this property should be left "
-		                      "blank, which allows NetworkManager to select the "
-		                      "appropriate settings automatically.",
-		                      NULL,
-		                      G_PARAM_READWRITE | NM_SETTING_PARAM_SERIALIZE));
+						  "Number",
+						  "Number to dial when establishing a PPP data session "
+						  "with the GSM-based mobile broadband network.  In most "
+						  "cases, leave the number blank and a number selecting "
+						  "the APN specified in the 'apn' property will be used "
+						  "automatically when required.",
+						  NULL,
+						  G_PARAM_READWRITE | NM_SETTING_PARAM_SERIALIZE));
 
 	/**
 	 * NMSettingGsm:username:
@@ -597,6 +513,20 @@ nm_setting_gsm_class_init (NMSettingGsmClass *setting_class)
 						  NULL,
 						  G_PARAM_READWRITE | NM_SETTING_PARAM_SERIALIZE | NM_SETTING_PARAM_SECRET));
 
+	/**
+	 * NMSettingGsm:password-flags:
+	 *
+	 * Flags indicating how to handle #NMSettingGsm:password:.
+	 **/
+	g_object_class_install_property (object_class, PROP_PASSWORD_FLAGS,
+		 g_param_spec_uint (NM_SETTING_GSM_PASSWORD_FLAGS,
+		                    "Password Flags",
+		                    "Flags indicating how to handle the GSM password.",
+		                    NM_SETTING_SECRET_FLAG_NONE,
+		                    NM_SETTING_SECRET_FLAGS_ALL,
+		                    NM_SETTING_SECRET_FLAG_NONE,
+		                    G_PARAM_READWRITE | NM_SETTING_PARAM_SERIALIZE));
+
 	/**
 	 * NMSettingGsm:apn:
 	 *
@@ -651,9 +581,9 @@ nm_setting_gsm_class_init (NMSettingGsmClass *setting_class)
 	 * NMSettingGsm:network-type:
 	 *
 	 * Network preference to force the device to only use specific network
-	 * technologies.  Permitted values are those specified by
-	 * #NMSettingGsmNetworkType.  Note that not all devices allow network
-	 * preference control.
+	 * technologies.  The permitted values are: -1: any, 0: 3G only,
+	 * 1: GPRS/EDGE only, 2: prefer 3G, and 3: prefer 2G.  Note that not all
+	 * devices allow network preference control.
 	 **/
 	g_object_class_install_property
 		(object_class, PROP_NETWORK_TYPE,
@@ -673,8 +603,7 @@ nm_setting_gsm_class_init (NMSettingGsmClass *setting_class)
 	 * NMSettingGsm:allowed-bands:
 	 *
 	 * Bitfield of allowed frequency bands.  Note that not all devices allow
-	 * frequency band control.  Permitted values are those specified by
-	 * #NMSettingGsmNetworkBand.
+	 * frequency band control.
 	 **/
 	g_object_class_install_property
 		(object_class, PROP_ALLOWED_BANDS,
@@ -696,8 +625,7 @@ nm_setting_gsm_class_init (NMSettingGsmClass *setting_class)
 		                     || NM_SETTING_GSM_BAND_U850
 		                     || NM_SETTING_GSM_BAND_U900
 		                     || NM_SETTING_GSM_BAND_U17IX
-		                     || NM_SETTING_GSM_BAND_U1900
-		                     || NM_SETTING_GSM_BAND_U2600,
+		                     || NM_SETTING_GSM_BAND_U1900,
 		                    NM_SETTING_GSM_BAND_ANY,
 		                    G_PARAM_READWRITE | G_PARAM_CONSTRUCT | NM_SETTING_PARAM_SERIALIZE));
 
@@ -718,6 +646,20 @@ nm_setting_gsm_class_init (NMSettingGsmClass *setting_class)
 						  NULL,
 						  G_PARAM_READWRITE | NM_SETTING_PARAM_SERIALIZE | NM_SETTING_PARAM_SECRET));
 
+	/**
+	 * NMSettingGsm:pin-flags:
+	 *
+	 * Flags indicating how to handle #NMSettingGsm:pin:.
+	 **/
+	g_object_class_install_property (object_class, PROP_PIN_FLAGS,
+		 g_param_spec_uint (NM_SETTING_GSM_PIN_FLAGS,
+		                    "PIN Flags",
+		                    "Flags indicating how to handle the GSM SIM PIN.",
+		                    NM_SETTING_SECRET_FLAG_NONE,
+		                    NM_SETTING_SECRET_FLAGS_ALL,
+		                    NM_SETTING_SECRET_FLAG_NONE,
+		                    G_PARAM_READWRITE | NM_SETTING_PARAM_SERIALIZE));
+
 	/**
 	 * NMSettingGsm:home-only:
 	 *
@@ -733,31 +675,4 @@ nm_setting_gsm_class_init (NMSettingGsmClass *setting_class)
 						  "not be made.",
 						  FALSE,
 						  G_PARAM_READWRITE | NM_SETTING_PARAM_SERIALIZE));
-
-	/* Deprecated properties */
-	/**
-	 * NMSettingGsm:puk:
-	 *
-	 * DEPRECATED
-	 **/
-	g_object_class_install_property
-		(object_class, PROP_PUK,
-		 g_param_spec_string (NM_SETTING_GSM_PUK,
-						  "PUK (DEPRECATED and UNUSED)",
-						  "PUK (DEPRECATED and UNUSED)",
-						  NULL,
-						  G_PARAM_READWRITE | NM_SETTING_PARAM_SERIALIZE | NM_SETTING_PARAM_SECRET));
-
-	/**
-	 * NMSettingGsm:band:
-	 *
-	 * DEPRECATED
-	 **/
-	g_object_class_install_property
-		(object_class, PROP_BAND,
-		 g_param_spec_int (NM_SETTING_GSM_BAND,
-		                    "Band (DEPRECATED and UNUSED)",
-		                    "Band (DEPRECATED and UNUSED)",
-		                    -1, 5, -1,
-		                    G_PARAM_READWRITE | G_PARAM_CONSTRUCT | NM_SETTING_PARAM_SERIALIZE));
 }
