@@ -19,7 +19,7 @@
  * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
  * Boston, MA 02110-1301 USA.
  *
- * (C) Copyright 2007 - 2008 Red Hat, Inc.
+ * (C) Copyright 2007 - 2011 Red Hat, Inc.
  * (C) Copyright 2007 - 2008 Novell, Inc.
  */
 
@@ -39,14 +39,8 @@ G_BEGIN_DECLS
 
 #define NM_SETTING_VPN_SETTING_NAME "vpn"
 
-/**
- * NMSettingVpnError:
- * @NM_SETTING_VPN_ERROR_UNKNOWN: unknown or unclassified error
- * @NM_SETTING_VPN_ERROR_INVALID_PROPERTY: the property was invalid
- * @NM_SETTING_VPN_ERROR_MISSING_PROPERTY: the property was missing and is
- * required
- */
-typedef enum {
+typedef enum
+{
 	NM_SETTING_VPN_ERROR_UNKNOWN = 0,
 	NM_SETTING_VPN_ERROR_INVALID_PROPERTY,
 	NM_SETTING_VPN_ERROR_MISSING_PROPERTY,
@@ -77,14 +71,7 @@ typedef struct {
 	void (*_reserved4) (void);
 } NMSettingVPNClass;
 
-/**
- * VPNIterFunc:
- * @key: the name of the data or secret item
- * @value: the value of the data or secret item
- * @user_data: User data passed to nm_setting_vpn_foreach_data_item() or
- * nm_setting_vpn_foreach_secret()
- **/
-typedef void (*VPNIterFunc) (const char *key, const char *value, gpointer user_data);
+typedef void (*NMVPNIterFunc) (const char *key, const char *value, gpointer user_data);
 
 GType nm_setting_vpn_get_type (void);
 
@@ -100,7 +87,7 @@ const char *      nm_setting_vpn_get_data_item     (NMSettingVPN *setting,
 void              nm_setting_vpn_remove_data_item  (NMSettingVPN *setting,
                                                     const char *key);
 void              nm_setting_vpn_foreach_data_item (NMSettingVPN *setting,
-                                                    VPNIterFunc func,
+                                                    NMVPNIterFunc func,
                                                     gpointer user_data);
 
 void              nm_setting_vpn_add_secret        (NMSettingVPN *setting,
@@ -111,7 +98,7 @@ const char *      nm_setting_vpn_get_secret        (NMSettingVPN *setting,
 void              nm_setting_vpn_remove_secret     (NMSettingVPN *setting,
                                                     const char *key);
 void              nm_setting_vpn_foreach_secret    (NMSettingVPN *setting,
-                                                    VPNIterFunc func,
+                                                    NMVPNIterFunc func,
                                                     gpointer user_data);
 
 G_END_DECLS
