@@ -30,6 +30,7 @@
 #include "NetworkManagerUtils.h"
 #include "nm-marshal.h"
 #include "nm-logging.h"
+#include "nm-system.h"
 
 /* Pre-DHCP addrconf timeout, in seconds */
 #define NM_IP6_TIMEOUT 20
@@ -155,7 +156,7 @@ nm_ip6_device_new (NMIP6Manager *manager, int ifindex)
 	}
 
 	device->ifindex = ifindex;
-	device->iface = nm_netlink_index_to_iface (ifindex);
+	device->iface = g_strdup (nm_netlink_index_to_iface (ifindex));
 	if (!device->iface) {
 		nm_log_err (LOGD_IP6, "(%d): could not find interface name from index.",
 		            ifindex);
@@ -260,7 +261,7 @@ static gboolean
 rdnss_expired (gpointer user_data)
 {
 	NMIP6Device *device = user_data;
-	CallbackInfo info = { device, IP6_DHCP_OPT_NONE, FALSE };
+	CallbackInfo info = { device, IP6_DHCP_OPT_NONE };
 
 	nm_log_dbg (LOGD_IP6, "(%s): IPv6 RDNSS information expired", device->iface);
 
@@ -307,9 +308,9 @@ set_rdnss_timeout (NMIP6Device *device)
 	}
 
 	if (expires) {
-		device->rdnss_timeout_id = g_timeout_add_seconds (MIN (expires - now, G_MAXUINT32 - 1),
-		                                                  rdnss_expired,
-		                                                  device);
+		device->rdnss_timeout_id = g_timeout_add_seconds (expires - now,
+														  rdnss_expired,
+														  device);
 	}
 }
 
@@ -362,7 +363,7 @@ set_dnssl_timeout (NMIP6Device *device)
 	}
 
 	if (expires) {
-		device->dnssl_timeout_id = g_timeout_add_seconds (MIN (expires - now, G_MAXUINT32 - 1),
+		device->dnssl_timeout_id = g_timeout_add_seconds (expires - now,
 		                                                  dnssl_expired,
 		                                                  device);
 	}
@@ -688,12 +689,9 @@ process_nduseropt_rdnss (NMIP6Device *device, struct nd_opt_hdr *opt)
 	for (addr = (struct in6_addr *) (rdnss_opt + 1); opt_len >= 2; addr++, opt_len -= 2) {
 		char buf[INET6_ADDRSTRLEN + 1];
 
-		if (!inet_ntop (AF_INET6, addr, buf, sizeof (buf))) {
-			nm_log_warn (LOGD_IP6, "(%s): received invalid RA-provided nameserver", device->iface);
-			continue;
-		}
+		if (!inet_ntop (AF_INET6, addr, buf, sizeof (buf)))
+			strcpy(buf, "[invalid]");
 
-		/* Update the cached timeout if we already saw this server */
 		for (i = 0; i < device->rdnss_servers->len; i++) {
 			cur_server = &(g_array_index (device->rdnss_servers, NMIP6RDNSS, i));
 
@@ -703,7 +701,7 @@ process_nduseropt_rdnss (NMIP6Device *device, struct nd_opt_hdr *opt)
 			cur_server->expires = server.expires;
 
 			if (server.expires > 0) {
-				nm_log_dbg (LOGD_IP6, "(%s): refreshing RA-provided nameserver %s (expires in %ld seconds)",
+				nm_log_dbg (LOGD_IP6, "(%s): refreshing RA-provided nameserver %s (expires in %d seconds)",
 				            device->iface, buf,
 				            server.expires - now);
 				break;
@@ -722,7 +720,7 @@ process_nduseropt_rdnss (NMIP6Device *device, struct nd_opt_hdr *opt)
 		if (i < device->rdnss_servers->len)
 			continue;
 
-		nm_log_dbg (LOGD_IP6, "(%s): found RA-provided nameserver %s (expires in %ld seconds)",
+		nm_log_dbg (LOGD_IP6, "(%s): found RA-provided nameserver %s (expires in %d seconds)",
 		            device->iface, buf, server.expires - now);
 
 		server.addr = *addr;
@@ -836,7 +834,6 @@ process_nduseropt_dnssl (NMIP6Device *device, struct nd_opt_hdr *opt)
 		if (domain_str[0] == '\0')
 			continue;
 
-		/* Update cached domain information if we've seen this domain before */
 		for (i = 0; i < device->dnssl_domains->len; i++) {
 			cur_domain = &(g_array_index (device->dnssl_domains, NMIP6DNSSL, i));
 
@@ -846,7 +843,7 @@ process_nduseropt_dnssl (NMIP6Device *device, struct nd_opt_hdr *opt)
 			cur_domain->expires = domain.expires;
 
 			if (domain.expires > 0) {
-				nm_log_dbg (LOGD_IP6, "(%s): refreshing RA-provided domain %s (expires in %ld seconds)",
+				nm_log_dbg (LOGD_IP6, "(%s): refreshing RA-provided domain %s (expires in %d seconds)",
 				            device->iface, domain_str,
 				            domain.expires - now);
 				break;
@@ -865,7 +862,7 @@ process_nduseropt_dnssl (NMIP6Device *device, struct nd_opt_hdr *opt)
 		if (i < device->dnssl_domains->len)
 			continue;
 
-		nm_log_dbg (LOGD_IP6, "(%s): found RA-provided domain %s (expires in %ld seconds)",
+		nm_log_dbg (LOGD_IP6, "(%s): found RA-provided domain %s (expires in %d seconds)",
 		            device->iface, domain_str, domain.expires - now);
 
 		g_assert (strlen (domain_str) < sizeof (domain.domain));
@@ -1087,7 +1084,7 @@ nm_ip6_manager_prepare_interface (NMIP6Manager *manager,
 		nm_utils_do_sysctl (accept_ra_path, "0\n");
 	} else {
 		device->target_state = NM_IP6_DEVICE_GOT_ADDRESS;
-		nm_utils_do_sysctl (accept_ra_path, "2\n");
+		nm_utils_do_sysctl (accept_ra_path, "1\n");
 	}
 
 	return TRUE;
@@ -1337,7 +1334,6 @@ nm_ip6_manager_init (NMIP6Manager *manager)
 	priv->monitor = nm_netlink_monitor_get ();
 	nm_netlink_monitor_subscribe (priv->monitor, RTNLGRP_IPV6_IFADDR, NULL);
 	nm_netlink_monitor_subscribe (priv->monitor, RTNLGRP_IPV6_PREFIX, NULL);
-	nm_netlink_monitor_subscribe (priv->monitor, RTNLGRP_IPV6_ROUTE, NULL);
 	nm_netlink_monitor_subscribe (priv->monitor, RTNLGRP_ND_USEROPT, NULL);
 	nm_netlink_monitor_subscribe (priv->monitor, RTNLGRP_LINK, NULL);
 
