@@ -22,19 +22,17 @@
 #define _GNU_SOURCE
 #include <dlfcn.h>
 #include <syslog.h>
+#include <execinfo.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
 #include <errno.h>
 #include <sys/wait.h>
 #include <sys/stat.h>
+#include <execinfo.h>
 #include <strings.h>
 #include <string.h>
 
-#ifdef ENABLE_CRASHTRACE
-#include <execinfo.h>
-#endif
-
 #include <glib/gi18n.h>
 
 #include "nm-logging.h"
@@ -43,8 +41,9 @@ static guint32 log_level = LOGL_INFO | LOGL_WARN | LOGL_ERR;
 static guint32 log_domains = \
 	LOGD_HW | LOGD_RFKILL | LOGD_ETHER | LOGD_WIFI | LOGD_BT | LOGD_MB | \
 	LOGD_DHCP4 | LOGD_DHCP6 | LOGD_PPP | LOGD_IP4 | LOGD_IP6 | LOGD_AUTOIP4 | \
-	LOGD_DNS | LOGD_VPN | LOGD_SHARING | LOGD_SUPPLICANT | LOGD_USER_SET | \
-	LOGD_SYS_SET | LOGD_SUSPEND | LOGD_CORE | LOGD_DEVICE | LOGD_OLPC_MESH;
+	LOGD_DNS | LOGD_VPN | LOGD_SHARING | LOGD_SUPPLICANT | LOGD_AGENTS | \
+	LOGD_SETTINGS | LOGD_SUSPEND | LOGD_CORE | LOGD_DEVICE | LOGD_OLPC_MESH | \
+	LOGD_WIMAX;
 
 typedef struct {
 	guint32 num;
@@ -78,12 +77,13 @@ static const LogDesc domain_descs[] = {
 	{ LOGD_VPN,       "VPN" },
 	{ LOGD_SHARING,   "SHARING" },
 	{ LOGD_SUPPLICANT,"SUPPLICANT" },
-	{ LOGD_USER_SET,  "USER_SET" },
-	{ LOGD_SYS_SET,   "SYS_SET" },
+	{ LOGD_AGENTS,    "AGENTS" },
+	{ LOGD_SETTINGS,  "SETTINGS" },
 	{ LOGD_SUSPEND,   "SUSPEND" },
 	{ LOGD_CORE,      "CORE" },
 	{ LOGD_DEVICE,    "DEVICE" },
 	{ LOGD_OLPC_MESH, "OLPC" },
+	{ LOGD_WIMAX,     "WIMAX" },
 	{ 0, NULL }
 };
 
@@ -217,13 +217,12 @@ nm_logging_level_enabled (guint32 level)
 	return !!(log_level & level);
 }
 
-void
-_nm_log (const char *loc,
-         const char *func,
-         guint32 domain,
-         guint32 level,
-         const char *fmt,
-         ...)
+void _nm_log (const char *loc,
+              const char *func,
+              guint32 domain,
+              guint32 level,
+              const char *fmt,
+              ...)
 {
 	va_list args;
 	char *msg;
@@ -255,7 +254,6 @@ _nm_log (const char *loc,
 static void
 fallback_get_backtrace (void)
 {
-#ifdef ENABLE_CRASHTRACE
 	void *frames[64];
 	Dl_info info;
 	size_t size;
@@ -283,7 +281,6 @@ fallback_get_backtrace (void)
 		}
 	}
 	syslog (LOG_CRIT, "******************* END **********************************");
-#endif /* ENABLE_CRASHTRACE */
 }
 
 
