@@ -15,7 +15,7 @@
  * with this program; if not, write to the Free Software Foundation, Inc.,
  * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
- * (C) Copyright 2005 - 2008 Red Hat, Inc.
+ * (C) Copyright 2005 - 2010 Red Hat, Inc.
  */
 
 #ifndef NM_ACTIVATION_REQUEST_H
@@ -37,29 +37,10 @@ typedef struct {
 	GObject parent;
 } NMActRequest;
 
-typedef enum {
-	SECRETS_CALLER_NONE = 0,
-	SECRETS_CALLER_ETHERNET,
-	SECRETS_CALLER_WIFI,
-	SECRETS_CALLER_GSM,
-	SECRETS_CALLER_CDMA,
-	SECRETS_CALLER_PPP,
-	SECRETS_CALLER_HSO_GSM
-} RequestSecretsCaller;
-
 typedef struct {
 	GObjectClass parent;
 
 	/* Signals */
-	void (*connection_secrets_updated)  (NMActRequest *req,
-	                                     NMConnection *connection,
-	                                     GSList *updated_settings,
-	                                     RequestSecretsCaller caller);
-	void (*connection_secrets_failed)   (NMActRequest *req,
-	                                     NMConnection *connection,
-	                                     const char *setting,
-	                                     RequestSecretsCaller caller);
-
 	void (*properties_changed) (NMActRequest *req, GHashTable *properties);
 } NMActRequestClass;
 
@@ -68,15 +49,11 @@ GType nm_act_request_get_type (void);
 NMActRequest *nm_act_request_new          (NMConnection *connection,
                                            const char *specific_object,
                                            gboolean user_requested,
+                                           gulong user_uid,
+                                           gboolean assumed,
                                            gpointer *device);  /* An NMDevice */
 
 NMConnection *nm_act_request_get_connection     (NMActRequest *req);
-gboolean      nm_act_request_request_connection_secrets (NMActRequest *req,
-                                                         const char *setting_name,
-                                                         gboolean request_new,
-                                                         RequestSecretsCaller caller,
-                                                         const char *hint1,
-                                                         const char *hint2);
 const char *  nm_act_request_get_specific_object (NMActRequest *req);
 
 void          nm_act_request_set_specific_object (NMActRequest *req,
@@ -90,6 +67,10 @@ void          nm_act_request_set_default (NMActRequest *req, gboolean is_default
 
 gboolean      nm_act_request_get_default (NMActRequest *req);
 
+void          nm_act_request_set_default6 (NMActRequest *req, gboolean is_default6);
+
+gboolean      nm_act_request_get_default6 (NMActRequest *req);
+
 gboolean      nm_act_request_get_shared (NMActRequest *req);
 
 void          nm_act_request_set_shared (NMActRequest *req, gboolean shared);
@@ -100,4 +81,33 @@ void          nm_act_request_add_share_rule (NMActRequest *req,
 
 GObject *     nm_act_request_get_device (NMActRequest *req);
 
+gboolean      nm_act_request_get_assumed (NMActRequest *req);
+
+/* Secrets handling */
+
+typedef void (*NMActRequestSecretsFunc) (NMActRequest *req,
+                                         guint32 call_id,
+                                         NMConnection *connection,
+                                         GError *error,
+                                         gpointer user_data);
+
+/* NOTE: these values should match the NM_SECRET_AGENT_GET_SECRETS_FLAGS in
+ * the nm-secret-agent.xml introspection file.
+ */
+enum {
+	NM_ACT_REQUEST_GET_SECRETS_FLAG_NONE = 0x0,
+	NM_ACT_REQUEST_GET_SECRETS_FLAG_ALLOW_INTERACTION = 0x1,
+	NM_ACT_REQUEST_GET_SECRETS_FLAG_REQUEST_NEW = 0x2
+};
+
+guint32 nm_act_request_get_secrets (NMActRequest *req,
+                                    const char *setting_name,
+                                    guint32 flags,
+                                    const char *hint,
+                                    NMActRequestSecretsFunc callback,
+                                    gpointer callback_data);
+
+void nm_act_request_cancel_secrets (NMActRequest *req, guint32 call_id);
+
 #endif /* NM_ACTIVATION_REQUEST_H */
+
