@@ -15,9 +15,10 @@
  * with this program; if not, write to the Free Software Foundation, Inc.,
  * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
- * Copyright (C) 2009 - 2010 Red Hat, Inc.
+ * Copyright (C) 2009 - 2011 Red Hat, Inc.
  */
 
+#include <config.h>
 #include <signal.h>
 #include <string.h>
 #include <stdlib.h>
@@ -38,6 +39,9 @@
 #include "nm-device-wifi.h"
 #include "nm-device-olpc-mesh.h"
 #include "nm-device-ethernet.h"
+#if WITH_WIMAX
+#include "nm-device-wimax.h"
+#endif
 
 typedef struct {
 	GUdevClient *client;
@@ -88,7 +92,7 @@ rfkill_type_to_desc (RfKillType rtype)
 	if (rtype == 0)
 		return "WiFi";
 	else if (rtype == 1)
-		return "WWAN";
+		return "WWan";
 	else if (rtype == 2)
 		return "WiMAX";
 	return "unknown";
@@ -335,6 +339,15 @@ is_olpc_mesh (GUdevDevice *device)
 	return (prop != NULL);
 }
 
+static gboolean
+is_wimax (const char *driver)
+{
+	/* FIXME: check 'DEVTYPE' instead; but since we only support Intel
+	 * WiMAX devices for now this is appropriate.
+	 */
+	return g_strcmp0 (driver, "i2400m_usb") == 0;
+}
+
 static GObject *
 device_creator (NMUdevManager *manager,
                 GUdevDevice *udev_device,
@@ -361,13 +374,9 @@ device_creator (NMUdevManager *manager,
 		if (parent) {
 			driver = g_udev_device_get_driver (parent);
 			if (!driver) {
-				/* try the grandparent if it's an ibmebus device or if the
-				 * subsys is NULL which usually indicates some sort of
-				 * platform device like a 'gadget' net interface.
-				 */
+				/* try the grandparent only if it's an ibmebus device */
 				subsys = g_udev_device_get_subsystem (parent);
-				if (   (g_strcmp0 (subsys, "ibmebus") == 0)
-				    || (subsys == NULL)) {
+				if (subsys && !strcmp (subsys, "ibmebus")) {
 					grandparent = g_udev_device_get_parent (parent);
 					if (grandparent)
 						driver = g_udev_device_get_driver (grandparent);
@@ -377,12 +386,8 @@ device_creator (NMUdevManager *manager,
 	}
 
 	if (!driver) {
-		if (g_str_has_prefix (ifname, "easytether")) {
-			driver = "easytether";
-		} else {
-			nm_log_warn (LOGD_HW, "%s: couldn't determine device driver; ignoring...", path);
-			goto out;
-		}
+		nm_log_warn (LOGD_HW, "%s: couldn't determine device driver; ignoring...", path);
+		goto out;
 	}
 
 	ifindex = g_udev_device_get_sysfs_attr_as_int (udev_device, "ifindex");
@@ -395,7 +400,11 @@ device_creator (NMUdevManager *manager,
 		device = (GObject *) nm_device_olpc_mesh_new (path, ifname, driver);
 	else if (is_wireless (udev_device))
 		device = (GObject *) nm_device_wifi_new (path, ifname, driver);
-	else
+	else if (is_wimax (driver)) {
+#if WITH_WIMAX
+		device = (GObject *) nm_device_wimax_new (path, ifname, driver);
+#endif
+	} else
 		device = (GObject *) nm_device_ethernet_new (path, ifname, driver);
 
 out:
@@ -411,27 +420,14 @@ net_add (NMUdevManager *self, GUdevDevice *device)
 {
 	gint etype;
 	const char *iface;
-	const char *tmp;
-	gboolean is_ctc;
+	const char *devtype;
 
 	g_return_if_fail (device != NULL);
 
-	iface = g_udev_device_get_name (device);
-	if (!iface) {
-		nm_log_dbg (LOGD_HW, "failed to get device's interface");
-		return;
-	}
-
 	etype = g_udev_device_get_sysfs_attr_as_int (device, "type");
-	is_ctc = (strncmp (iface, "ctc", 3) == 0) && (etype == 256);
-
-	/* Ignore devices that don't report Ethernet encapsulation, except for
-	 * s390 CTC-type devices that report 256 for some reason.
-	 * FIXME: use something other than interface name to detect CTC here.
-	 */
-	if ((etype != 1) && (is_ctc == FALSE)) {
+	if (etype != 1) {
 		nm_log_dbg (LOGD_HW, "ignoring interface with type %d", etype);
-		return;
+		return; /* Not using ethernet encapsulation, don't care */
 	}
 
 	/* Not all ethernet devices are immediately usable; newer mobile broadband
@@ -441,22 +437,16 @@ net_add (NMUdevManager *self, GUdevDevice *device)
 	 * subclass.  ModemManager will pick it up though and so we'll handle it
 	 * through the mobile broadband stuff.
 	 */
-	tmp = g_udev_device_get_property (device, "DEVTYPE");
-	if (g_strcmp0 (tmp, "wwan") == 0) {
-		nm_log_dbg (LOGD_HW, "ignoring interface with devtype '%s'", tmp);
+	devtype = g_udev_device_get_property (device, "DEVTYPE");
+	if (devtype && !strcmp (devtype, "wwan")) {
+		nm_log_dbg (LOGD_HW, "ignoring interface with devtype '%s'", devtype);
 		return;
 	}
 
-	/* Ignore Nokia cdc-ether interfaces in PC-Suite mode since we need to
-	 * talk phonet to use them, which ModemManager doesn't do yet.
-	 */
-	tmp = g_udev_device_get_property (device, "ID_VENDOR_ID");
-	if (g_strcmp0 (tmp, "0421") == 0) { /* Nokia vendor ID */
-		tmp = g_udev_device_get_property (device, "ID_MODEL");
-		if (tmp && (strstr (tmp, "PC-Suite") || strstr (tmp, "PC Suite"))) {
-			nm_log_dbg (LOGD_HW, "ignoring Nokia PC-Suite ethernet interface");
-			return;
-		}
+	iface = g_udev_device_get_name (device);
+	if (!iface) {
+		nm_log_dbg (LOGD_HW, "failed to get device's interface");
+		return;
 	}
 
 	g_signal_emit (self, signals[DEVICE_ADDED], 0, device, device_creator);
